# -*- coding: utf-8 -*-
#

# import des librairies
import pygame
import sys
import math

from time import sleep #pour rendre transparent côté élève l'utilisation de "sleep"
from couleurs import rgb


def creer_fenetre(largeur=200, hauteur=300, titre="Fenetre graphique"):
    '''
    alias de fenetre()
    '''
    fenetre(largeur, hauteur, titre)

def window(largeur=200, hauteur=300, titre="Fenetre graphique"):
    '''
    alias de fenetre()
    '''
    fenetre(largeur, hauteur, titre)

def fenetre(largeur=200, hauteur=300, titre="Fenetre graphique"):
    """
    Crée et affiche une fenêtre graphique.

    Arguments:
        largeur (int, optionel): Largeur de la fenetre en pixels (``200`` par défaut)
        hauteur (int, optionel): Hauteur de la fenetre en pixels (``300`` par défaut)
        titre (str, optionel): Titre de la fenetre (``Fenetre graphique`` par défaut)
    """

    pygame.init()
    global fenetre
    fenetre = pygame.display.set_mode((largeur, hauteur))
    pygame.display.set_caption(titre)
    # active la répétition des touches
    pygame.key.set_repeat(1)
    fenetre.fill(rgb('blanc'))
    pygame.display.update()


def ecoute_evenements():
    '''
    alias de demande_evenements()
    '''
    demande_evenements()

def events():
    '''
    alias de demande_evenements()
    '''
    demande_evenements()

def demande_evenements():
    """
    Récupère les évenements pygame gère la fermeture de la fenetre et retourne les évenements formatés.

    Renvoie un dictionnaire d'évenements formaté comme suit:
    ``{'touche1': None, 'touche2':None, 'souris': [x,y], 'click': [x,y]}``

    Les valeurs ``None`` pour les touches peuvent surprendre mais il est nécéssaire d'utiliser un dictionnaire pour avoir les coordonnées
    éventuelles de la souris lors d'un click par exemple. Pour les touches clavier, l'importance est la présence de la cléf et la valeur associée est donc ``None``.

    - Les caractères alphanumériques sont encodés en ascii (``'a'``, ``'n'``, ``';'``) et, si présent, leur valeur est ``None``.
    - les touches spéciales ont les clefs ``'espace'``, ``'haut'``, ``'bas'``, ``'droite'``, ``'gauche'`` et, si présent, leur valeur est ``None``.
    - Un clic avec le bouton gauche de la souris ajoute une clef ``'clic'``. Sa valeur est une liste ``[x, y]`` des coordonnées de la souris.
    - Un déplacement de la souris ajoute une clef ``'souris'``. Sa valeur est une liste ``[x, y]`` des coordonnées de la souris.

    """

    # Initialisation du dictionnaire de sortie
    evenements = {}

    touches_speciales = {
        'haut': pygame.K_UP,
        'bas': pygame.K_DOWN,
        'gauche': pygame.K_LEFT,
        'droite': pygame.K_RIGHT,
        'espace': pygame.K_SPACE
    }

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            sys.exit()
        elif event.type == pygame.KEYDOWN:
            for c, v in touches_speciales.items():
                # On parcourt le dictionnaire par clef, valeur
                # Par exmple c = 'haut' et v = pygame.K_UP
                if event.key == v:
                    # L'important est simplement d'avoir la clef dans le dictionnaire.
                    # La valeur n'a pas d'importance et est donc None
                    evenements[c] = None

            # Si l'unicode peut être convertit en ascii, on ajoute la clef au dictionnaire sous unicode
            if event.unicode.encode('ascii', 'ignore'):
                evenements[event.unicode] = None

        # Gestion des evenements souris
        elif event.type == pygame.MOUSEMOTION:
            evenements['souris'] = list(event.pos)
        elif event.type == pygame.MOUSEBUTTONDOWN:
            evenements['clic'] = list(event.pos)

    return evenements


def trace_cercle(x, y, couleur='bleu', rayon=5, epaisseur=0):
    '''
    alias de cercle()
    '''
    cercle(x, y, couleur , rayon, epaisseur)

def circle(x, y, couleur='bleu', rayon=5, epaisseur=0):
    '''
    alias de cercle()
    '''
    cercle(x, y, couleur , rayon, epaisseur)

def cercle(x, y, couleur='bleu', rayon=5, epaisseur=0):
    """
    Trace un cercle dans la fenetre graphique.

    Arguments:
        x (int): Abscisse du centre du cercle
        y (int): Ordonnée du centre du cercle
        rayon (int, optionel): Rayon du cercle (5 par défaut)
        epaisseur (int, optionel): Epaisseur du cercle (``0`` par défaut). Si ``0``, le cercle sera rempli et apparaitra comme un disque.
        couleur (:ref:`couleur <couleur>`, optionnel): Couleur du cercle (bleu par défaut)
    """
    couleur = rgb(couleur)
    pygame.draw.circle(fenetre, couleur, (x,y), rayon, epaisseur)
    pygame.display.update()


def trace_point(x, y, couleur='bleu'):
    '''
    alias de point()
    '''
    point(x, y, couleur)

def point(x, y, couleur='bleu'):
    """
    Trace un point dans la fenetre graphique.

    Arguments:
        x (int): Abscisse du point
        y (int): Ordonnée du point
        couleur (:ref:`couleur <couleur>`, optionnel): Couleur du point (bleu par défaut)
    """
    couleur = rgb(couleur)
    pygame.draw.circle(fenetre, couleur, (x,y), 1, 0)
    pygame.display.update()


def trace_rectangle(x, y, largeur, hauteur, couleur='bleu', epaisseur=1):
    '''
    alias de rectangle()
    '''
    rectangle(x, y, largeur, hauteur,couleur, epaisseur)

def rectangle(x, y, largeur, hauteur, couleur='bleu', epaisseur=1):
    """
    Trace un rectangle horizontal dans la fenetre graphique .

    Le sommet haut-gauche à pour coordonées ``(x,y)``, la ``largeur`` est la taille en abscisse
    et la ``hauteur`` la taille en ordonnée.

    Arguments:
        x (int): abscisse du sommet haut gauche du rectangle
        y (int): ordonnée du sommet haut gauche du rectangle
        largeur (int): taille du rectangle sur l'axe des abscisses
        hauteur (int): taille du rectangle sur l'axe des ordonnées
        couleur (:ref:`couleur <couleur>`, optionnel): Couleur du rectangle (``bleu`` par défaut)
        epaisseur (int, optionnel): Epaisseur des cotés du rectangle (``1`` par défaut). Si ``0``, le rectangle est rempli.

    """
    couleur = rgb(couleur)
    pygame.draw.rect(fenetre, couleur, (x, y, largeur, hauteur), epaisseur)
    pygame.display.update()


def trace_segment(x1, y1, x2, y2, couleur='bleu', epaisseur=1):
    '''
    alias de segment()
    '''
    segment(x1, y1, x2, y2, couleur, epaisseur)

def segment(x1, y1, x2, y2, couleur='bleu', epaisseur=1):
    """
    Trace un segment entre les points de coordonées ``(x1, y1)`` et ``(x2, y2)``.

    Arguments:
        x1 (int): abscisse de la première extremité du segment
        y1 (int): ordonnée de la première extremité du segment
        x2 (int): abscisse de la deuxieme extrémité du segment
        y2 (int): ordonnée de la deuxieme extrémité du segment
        couleur (:ref:`couleur <couleur>`, optionel): Couleur du segment (``bleu`` par défaut)
        epaisseur (int, optionel): Epaisseur du segment (``1`` par défaut)
    """
    couleur = rgb(couleur)
    pygame.draw.lines(fenetre, couleur, False, [(x1, y1), (x2, y2)], epaisseur)
    pygame.display.update()


def trace_vecteur(x, y, v, couleur='rouge', epaisseur=2):
    '''
    alias de vecteur()
    '''
    vecteur(x, y, v, couleur, epaisseur)

def vector(x, y, v, couleur='rouge', epaisseur=2):
    '''
    alias de vecteur()
    '''
    vecteur(x, y, v, couleur, epaisseur)

def vecteur(x, y, v, couleur='rouge', epaisseur=2):
    """
    Trace la représentation du vecteur ``v`` à partir du point d'origine ``(x, y)``.

    Arguments:
        x (int): abscisse du point d'origine de la représentation du vecteur
        y (int): ordonnée du point d'origine de la représentation du vecteur
        v (list): Coordonnées de la deuxieme extrémité du segment
        couleur (:ref:`couleur <couleur>`, optionel): Couleur du segment (``rouge`` par défaut)
        epaisseur (int, optionel): Epaisseur du segment (``1`` par défaut)
    """

    couleur = rgb(couleur)

    trace_segment(x, y, x+v[0], y+v[1], couleur, epaisseur)
    w1[0] = -.3 * math.cos(15 * math.pi / 180) * v[0] + .3 * math.sin(15 * math.pi/180) * (-v[1])
    w1[1] = -.3 * math.cos(15 * math.pi / 180) * v[1] + .3 * math.sin(15 * math.pi/180) * v[0]
    w2[0] = -.3 * math.cos(15 * math.pi / 180) * v[0] - .3 * math.sin(15 * math.pi/180) * (-v[1])
    w2[1] = -.3 * math.cos(15 * math.pi / 180) * v[1] - .3 * math.sin(15 * math.pi/180) * v[0]
    pygame.draw.polygon(
        fenetre,
        couleur,
        [
            (x + v[0], y + v[1]),
            (x + v[0] + w1[0], y + v[1] + w1[1]),
            (x + v[0] + w2[0], y + v[1] + w2[1]),
            (x + v[0], y + [1])
        ],
        0
    )
    pygame.display.update()

def trace_image(x, y, nom, largeur=50, hauteur=50):
    '''
    alias de image()
    '''
    image(x, y, nom, largeur, hauteur)

def image(x, y, nom, largeur=50, hauteur=50):
    """
    Trace une image dans la fenetre graphique.

    Arguments:
        x (int): Abscisse du centre de l'image
        y (int): Ordonnée du centre de l'image
        nom (str) : nom du fichier image (qui doit être dans le répertoire du script)
        largeur (int, optionel): Largeur de l'image (50 par défaut)
        hauteur (int, optionel): Hauteur de l'image (50 par défaut)
    """
    image = pygame.transform.scale(pygame.image.load(nom).convert_alpha(), (largeur, hauteur))
    fenetre.blit(image, (int(x-largeur/2),int(y-hauteur/2)))
    pygame.display.update()

def trace_explosion(x,y,couleur='orange',r=10,c=0.5,n=10):
    '''
    alias de explosion()
    '''
    explosion(x,y,couleur,r,c,n)

def explosion(x,y,couleur='orange',r=10,c=0.5,n=10):
    '''
    trace un polygône régulier étoilé à 2n côté,
    de rayon extérieur r,
    et tel que le rayon intérieur est égal à c*r
    (pour c=0, le polygône est réduit à n rayons du cencle de rayon r
    pour c=1, c'est un polygône régulier à 2n côtés)

    Arguments:
        x (int): Abscisse du centre de l'explosion
        y (int): Ordonnée du centre de l'explosion
        couleur (:ref:`couleur <couleur>`, optionel): Couleur (``orange`` par défaut)
        r (int): Rayon extérieur
        c (float):Coefficient pour obtenir le rayon intérieur égal à c*r
        n (int): Nombre de sommets
    '''
    couleur = rgb(couleur)
    pointlist=[]
    theta=2*math.pi/n
    for k in range(n):
        pointlist.append((x+r*math.cos(k*theta),y+r*math.sin(k*theta)))
        pointlist.append((x+c*r*math.cos((k+1/2)*theta),y+c*r*math.sin((k+1/2)*theta)))
    pointlist.append((x+r,y))
    pygame.draw.polygon(fenetre, couleur, pointlist)
    pygame.display.update()

def trace_axes(color='noir'):
    '''
    alias de axes()
    '''
    axes(color)

def axes(color='noir'):
    '''
    dessine les axes de coordonnées pour une meilleure compréhension par les élèves
    '''
    couleur = rgb(color)
    ymax=pygame.display.Info().current_h
    xmax=pygame.display.Info().current_w
    epaisseur=2
    pygame.draw.lines(fenetre, couleur, False, [(5, 0), (5, ymax)], epaisseur)
    pygame.draw.lines(fenetre, couleur, False, [(0, ymax-5), (5, ymax)], epaisseur)
    pygame.draw.lines(fenetre, couleur, False, [(10, ymax-5), (5, ymax)], epaisseur)
    pygame.draw.lines(fenetre, couleur, False, [(0, 5), (xmax, 5)], epaisseur)
    pygame.draw.lines(fenetre, couleur, False, [(xmax-5, 0), (xmax, 5)], epaisseur)
    pygame.draw.lines(fenetre, couleur, False, [(xmax-5, 10), (xmax, 5)], epaisseur)
    font=pygame.font.Font(None, 24,bold=False,italic=False)
    text=font.render(str(ymax),1,couleur)
    fenetre.blit(text, (15, ymax-30))
    text=font.render("y",1,couleur)
    fenetre.blit(text, (15, ymax-17))
    text=font.render(str(xmax),1,couleur)
    fenetre.blit(text, (xmax-35, 10))
    text=font.render("x",1,couleur)
    fenetre.blit(text, (xmax-15, 22))
    text=font.render("0",1,couleur)
    fenetre.blit(text, (10, 10))
    pygame.display.update()



"""
Ce module a ete ecrit par Olivier Brebant en aout 2011.

On peut l'utiliser librement sous licence MIT
"""

from tkinter import *
from math import floor
from couleurs import rgb, rgb2hex
global fenetre

class _Fenetre_graphique(Canvas):
    " Un canvas redimensionnable, avec repère usuel, zoomable... "
    # les coordonnées en pixels sont à l'intérieur du canvas (sans le bord)
    # R1 : repère lié aux pixels, B1 sa base associée
    # R2 : repère lié aux unités, B2 sa base associée

    def __init__(self, boss= None, largeur=400, hauteur=300,
                 xmin=-10, ymin=-10, xmax=10, ymax=10,
                 background='white', axes = True):
        Canvas.__init__(self,boss)
        self.configure(width=largeur, height=hauteur,
                       bg = background, border =5, relief=RIDGE)

        self.axes = axes
        self.xmin, self.ymin, self.xmax, self.ymax = xmin, ymin, xmax, ymax
        self.largeur = largeur #int(self.cget('width'))
        self.hauteur = hauteur #int(self.cget('height'))
        self.o2, self.p, self.inv_p = self._initialize_matrix(xmin,ymin,xmax,ymax,largeur,hauteur)
        self.pasU = self._reglage_pas()
        self.flag = 0           # pas de déplacement d'objets au départ
        border_w = self.winfo_reqwidth()-self.largeur   # largeur intérieure
        border_h = self.winfo_reqheight()-self.hauteur  # hauteur intérieure
        self.border=(border_w/2,border_h/2)             # taille du bord

        self.objets = []    # où l'on stocke les différents objets dessinés pour la réactualisation:
                            # [ tag, nature, xmin, ymin, xmax, ymax, couleur, epaisseur, remplissage ]
        self.redraw()

        self.bind('<Configure>', self._config)
        self.bind('<Button-1>', self.mouseDown)
        boss.bind('<Double-Button-1>', self.dblclic)
        self.bind('<Button1-Motion>', self.mouseMove)
        self.bind('<Button1-ButtonRelease>', self.mouseUp)
        self.bind('<Button-4>', self.wheel)  # roulette pour linux
        self.bind('<Button-5>', self.wheel)  # roulette pour linux
        boss.bind('<MouseWheel>', self.wheel) # pour windows
        self.master.protocol("WM_DELETE_WINDOW",self.fermeture)


    def _initialize_matrix(self,xmin,ymin,xmax,ymax,larg,haut):
        " initialise les matrices O2, P et inv_P "

        o2=[xmin*larg/(xmin-xmax),ymax*haut/(ymax-ymin)]      # coordonnées de O2 dans R1

        p = [[larg/(xmax-xmin),0],[0,haut/(ymin-ymax)]]        # matrice de passage de B1  à B2
        inv_p = [[(xmax-xmin)/larg,0],[0,(ymin-ymax)/haut]]    # matrice de passage de B2 à B1
        return o2,p,inv_p

    def _reglage_pas(self,pix=50,mult=2):
        # 50 pour environ tous les 50 pix, et 5 pour tous les k*5 unités
        cx = mult*floor(pix/mult/self.p[0][0]+0.5)
        cx = max(1,cx)
        cy = mult*floor(pix/mult/abs(self.p[1][1])+0.5)
        cy = max(1,cy)
        return (cx,cy)

    def inside(self, pt, zone):
        " décide si pt (2-tuple) est dans zone (4-tuple) "
        xmin, xmax = min(zone[0],zone[2]), max(zone[0],zone[2])
        ymin, ymax = min(zone[1],zone[3]), max(zone[1],zone[3])
        if pt[0]>xmin and pt[0]<xmax and pt[1]>ymin and pt[1]<ymax :
            return 1
        else :
            return 0

    def mouseDown(self, event):
        "Op. à effectuer quand le bouton gauche de la souris est enfoncé"
        # event.x et event.y contiennent les coordonnées du clic effectué (avec le bord !):
        self.x1, self.y1 = event.x-self.border[0], event.y-self.border[1] # on enlève le bord...

        zone_axeX = (0,self.o2[1]-20,self.largeur,self.o2[1]+20)
        zone_axeY = (self.o2[0]-20,0,self.o2[0]+20,self.hauteur)
        if self.axes:
            if self.inside((self.x1,self.y1),zone_axeX) :
                self.flag = 1       # drapeau de déplacement de l'axe X
            elif self.inside((self.x1,self.y1),zone_axeY):
                self.flag = 2       # drapeau de déplacement de l'ace Y
            else:
                self.flag = 3       # on déplace tout le repère
        else:
            self.flag = 3

    def dblclic(self,event):
        "Rend le repère orthonormé en se basant sur l'axe des X"
        self.p[1][1] = -self.p[0][0]
        self.inv_p[1][1] = 1/self.p[1][1]
        self._rafraichir()
        self.pasU = self._reglage_pas()
        self.redraw()

    def mouseMove(self, event):
        "Op. à effectuer quand la souris se déplace, bouton gauche enfoncé"
        x2,y2 = event.x-self.border[0], event.y-self.border[1]
        dx,dy = x2 - self.x1, y2 - self.y1

        if self.flag==1:
            self.translate(dx,0)

        elif self.flag==2:
            self.translate(0,dy)

        elif self.flag==3:
            self.translate(dx,dy)

        self.x1, self.y1 = x2, y2
        self.redraw()

    def mouseUp(self, event):
        "Op. à effectuer quand le bouton gauche de la souris est relâché"
        self.flag = 0

    def wheel(self,event):
        " Gestion du zoom sur action de la roulette " # attention ne fonctionne que sous Linux
        # event.num == 4 : roulette vers le haut
        # event.num == 5 : roulette vers le bas
        x,y = event.x-self.border[0], event.y-self.border[1] # x et y sont les coord. du clic a l'int. du canvas
        if event.num == 4 or event.delta>0:     # Zoom de +-10%
            zoom = 0.9
        else:
            zoom = 1.1
        zx = zy = zoom
        if (event.state == 20):  # si on appuie sur "Control" avec la roulette
            x , y = self.o2[0],self.o2[1]
        elif self.axes :
            if self.inside((x,y),(self.o2[0]-10,self.o2[1]-10,self.o2[0]+10,self.o2[1]+10)): # roulette proche du centre
                x,y = self.o2[0],self.o2[1]
            elif self.inside((x,y),(0,self.o2[1]-20,self.largeur,self.o2[1]+20)): # roulette sur l'axe des X
                zy = 1 # on bloque sur Y
                y = self.o2[1] # centrage sur l'ordonnée
            elif self.inside((x,y),(self.o2[0]-20,0,self.o2[0]+20,self.hauteur)): # roulette sur l'axe des Y
                zx = 1 # on bloque X
                x = self.o2[0] # centrage sur l'ordonnée

        self.zoom(zx,zy,x,y)   # on ajuste l'homothétie de coef (zx, zy) centré en (x,y)
        self.redraw()

    def translate(self,dx,dy):
        self.o2[0] += dx
        self.o2[1] += dy
        self._rafraichir()

    def zoom(self,zx,zy,x,y):
        self.o2[0]=zx*(self.o2[0]-x)+x
        self.o2[1]=zy*(self.o2[1]-y)+y
        self.p[0][0] = self.p[0][0]*zx
        self.p[1][1] = self.p[1][1]*zy
        self.inv_p[0][0] = 1/self.p[0][0]
        self.inv_p[1][1] = 1/self.p[1][1]
        self._rafraichir()
        self.pasU = self._reglage_pas()

    def pix2coord(self,xp,yp):
        xu = self.inv_p[0][0]*(xp-self.o2[0])
        yu = self.inv_p[1][1]*(yp-self.o2[1])
        return (xu,yu)

    def coord2pix(self,xu,yu):
        xp = self.p[0][0]*xu+self.o2[0]
        yp = self.p[1][1]*yu+self.o2[1]
        return (xp,yp)

    def _conv(self,x,y,delta=(0,0)):  # ajoute le bord et un décalage éventuel (delta) aux coordonnées en pixels
        tmp = self.coord2pix(x,y)
        return (tmp[0]+self.border[0]+delta[0],tmp[1]+self.border[1]+delta[1])

    def _rafraichir(self):
        self.xmax, self .ymax = self.pix2coord(self.largeur,0)
        self.xmin, self. ymin = self.pix2coord(0,self.hauteur)

    def _config(self, event): # event contient la taille du canvas, bord compris
        self.largeur = event.width-self.border[0]*2
        self.hauteur = event.height-self.border[1]*2
        self._rafraichir()
        self.configure(width=self.largeur, height=self.hauteur) # on met à jour les variables de dimension
        self.redraw() #dessine_axes()

    def dessine_axes(self):

        # axe X
        self.delete('axeX')

        self.create_line(self._conv(self.xmin,0,(10,0)),self._conv(self.xmax,0,(-10,0)), tag='axeX',arrow=LAST)
        self.create_text(self._conv(self.xmax,0), tag='axeX', text='x', anchor=NE)

        # Affichage des graduations 'intelligentes'
        # recherche des valeurs de l'axe qui doivent être affichées dans la fenêtre
        i1 = int(-floor(-self.xmin-20*self.inv_p[0][0])/self.pasU[0])
        i2 = int(floor(self.xmax-20*self.inv_p[0][0])/self.pasU[0])+1
        for i in range(i1,i2):
            p = self._conv(i*self.pasU[0],0)
            self.create_line(p[0],p[1]-4,p[0],p[1]+4,tag='axeX')
            if i==0 :
                self.create_text(p[0]-4,p[1]+4, text=str(int(i*self.pasU[0])),
                                 font=("Helvetica","8"), anchor = NE, tag='axeX')
            else:
                 self.create_text(p[0],p[1]+4, text=str(int(i*self.pasU[0])),
                                  font=("Helvetica","8"), anchor = N, tag='axeX')

        # axe Y
        self.delete('axeY')
        self.create_line(self._conv(0,self.ymin,(0,-10)),self._conv(0,self.ymax,(0,10)), tag='axeY',arrow=LAST)
        self.create_text(self._conv(0,self.ymax), tag='axeY', text='y', anchor=NE)

        # Affichage des graduations 'intelligentes'
        # recherche des valeurs de l'axe qui doivent être affichées dans la fenêtre
        i1 = int(floor(self.ymin+20*self.inv_p[1][1])/self.pasU[1])+1
        i2 = int(floor(self.ymax-20*self.inv_p[1][1])/self.pasU[1])
        for i in range(i1,i2):
            p = self._conv(0,i*self.pasU[1])
            self.create_line(p[0]-4,p[1],p[0]+4,p[1],tag='axeY')
            if i!=0 :
                 self.create_text(p[0]-4,p[1], text=str(int(i*self.pasU[1])),
                                  font=("Helvetica","8"), anchor = E, tag='axeY')


    def redraw(self):
        if self.axes :
            self.dessine_axes()
        for i in range(len(self.objets)):
            self.delete(self.objets[i][0])
            if self.objets[i][1]=='segment' :
                self.objets[i][0] = self.create_line(self._conv(self.objets[i][2],self.objets[i][3]),
                             self._conv(self.objets[i][4],self.objets[i][5]),
                             fill=self.objets[i][6], width=self.objets[i][7])
            elif self.objets[i][1]=='rectangle':
                self.objets[i][0] = self.create_rectangle(self._conv(self.objets[i][2],self.objets[i][3]),
                             self._conv(self.objets[i][4],self.objets[i][5]),
                             outline =self.objets[i][6], width=self.objets[i][7], fill=self.objets[i][8])
            elif self.objets[i][1]=='point':
                coef = 2
                pt = self._conv(self.objets[i][2],self.objets[i][3])
                taille = self.objets[i][5]
                if self.objets[i][6]=='rond':
                    self.objets[i][0] = self.create_oval(pt[0]-coef*taille,pt[1]-coef*taille,
                                                         pt[0]+coef*taille,pt[1]+coef*taille,
                                                         outline='', fill = self.objets[i][4])
                elif self.objets[i][6]=='croix':
                    self.delete(self.objets[i][7])
                    self.objets[i][0] = self.create_line(pt[0]-coef*taille,pt[1]-coef*taille,
                                                         pt[0]+coef*taille,pt[1]+coef*taille,
                                                         fill = self.objets[i][4])
                    self.objets[i][7] = self.create_line(pt[0]-coef*taille,pt[1]+coef*taille,
                                                         pt[0]+coef*taille,pt[1]-coef*taille,
                                                         fill = self.objets[i][4])
            elif self.objets[i][1]=='texte':
                self.objets[i][0]= self.create_text(self._conv(self.objets[i][2],self.objets[i][3]),
                                                    text=self.objets[i][4],fill=self.objets[i][5])


    def fermeture(self): # pour fermer proprement l'application avec la croix...
        #self.master.quit()
        self.master.destroy()


    def _trace_point(self,tab):
        pt = self._conv(self.objets[i][2],self.objets[i][3])

        self.objets[i][0] = self.create_oval(pt[0]-coef*taille,pt[1]-coef*taille,
                                                  pt[0]+coef*taille,pt[1]+coef*taille,
                                                  outline='', fill = self.objets[i][4])
    def loop(self):
        self.master.mainloop()

################################## Fin de la classe _Fenetre_graphique ##############

def trace_point(x,y, couleur='noir', taille=1, forme='rond'):
    '''
    alias de point()
    '''
    point(x,y, couleur, taille, forme)

def point(x,y, couleur='noir', taille=1, forme='rond'):
    """Ajoute un point dans la fenetre graphique aux coordonees ``(x, y)``.

    Arguments:
        x (float): abscisse du point
        y (float): ordonnée du point
        couleur (`couleur <#couleurs>`_, optionel): couleur du point (``noir`` par défaut)
        taille (int, optionel): taille du point (``1`` par défaut)
        forme (str, optionel): forme du point: rond/croix (``'rond'`` par défaut)
    """

    couleur = rgb2hex(rgb(couleur))
    global fenetre
    coef=2
    pt = fenetre._conv(x,y)
    tab = [0]*8
    if forme == 'rond':
        tab[0] = fenetre.create_oval(pt[0]-coef*taille,pt[1]-coef*taille,
                                  pt[0]+coef*taille,pt[1]+coef*taille,
                                  outline='', fill = couleur)
    elif forme == 'croix':
        tab[0] = fenetre.create_line(pt[0]-coef*taille,pt[1]-coef*taille,
                                  pt[0]+coef*taille,pt[1]+coef*taille,
                                  fill = couleur)
        tab[7] = fenetre.create_line(pt[0]-coef*taille,pt[1]+coef*taille,
                                  pt[0]+coef*taille,pt[1]-coef*taille,
                                  fill = couleur)

    tab[1] = 'point'
    tab[2] = x
    tab[3] = y
    tab[4] = couleur
    tab[5] = taille
    tab[6] = forme
    fenetre.objets.append(tab)

def trace_texte(x, y, texte, couleur='noir'):
    '''
    alias de texte()
    '''
    texte(x,y,texte,couleur)

def text(x, y, texte, couleur='noir'):
    '''
    alias de texte()
    '''
    texte(x,y,texte,couleur)

def texte(x, y,texte, couleur='noir'):
    """Trace un texte dans la fenêtre graphique au coordonées ``x, y``.

    Arguments:
        x (float): abscisse du point
        y (float): ordonnée du point
        texte (str): Texte à placer dans la fenêtre graphique
        couleur (`couleur <#couleurs>`_, optionel): Couleur du texte (``noir`` par défaut)

    """
    couleur = rgb2hex(rgb(couleur))

    global fenetre
    tab = [0]*6
    tab[0] = fenetre.create_text(x,y,text=texte,fill = couleur)
    tab[1] = 'texte'
    tab[2] = x
    tab[3] = y
    tab[4] = texte
    tab[5] = couleur
    fenetre.objets.append(tab)

def trace_segment(x1, y1, x2, y2, couleur='noir', taille=2):
    '''
    alias de segment()
    '''
    segment(x1, y1, x2, y2, couleur, taille)

def segment(x1, y1, x2, y2, couleur='noir', taille=2):
    """Trace un segment entre les points de coordonées ``(x1, y1)`` et ``(x2, y2)``.

    Arguments:
        x1,y1,x2,y2 (float): Coordonées des extrémités du segment.
        couleur (`couleur <#couleurs>`_, optionel): Couleur du segment (``noir`` par défaut).
        taille (int, optionl): Epaisseur du segment (``2`` par défaut).
        """
    couleur = rgb2hex(rgb(couleur))

    global fenetre

    tab = [0]*8
    tab[0] = fenetre.create_line(fenetre._conv(x1,y1),fenetre._conv(x2,y2),width=taille, fill = couleur)
    tab[1] = 'segment'
    tab[2] = x1
    tab[3] = y1
    tab[4] = x2
    tab[5] = y2
    tab[6] = couleur
    tab[7] = taille
    fenetre.objets.append(tab)

def trace_rectangle(x1, y1, largeur, hauteur, couleur='noir', taille=2, remplissage='jaune'):
    '''
    alias de rectangle()
    '''
    rectangle(x1, y1, largeur, hauteur, couleur, taille, remplissage)

def rectangle(x1, y1, largeur, hauteur, couleur='noir', taille=2, remplissage='jaune'):
    """Trace un rectangle dont le sommet en bas à gauche a pour coordonnées ``(x1, y1)``.

    Arguments:
        x1,y1 (float): Coordonnées du sommet en bas à gauche du rectangle.
        largeur,hauteur (float): Largeur et hauteur du rectangle
        couleur (`couleur <#couleurs>`_, optionel): Couleur des cotés du rectangle (``noir`` par défaut).
        taille (int, optionel): épaisseur des cotés du rectangle. ( ``2`` par défaut).
        remplissage (str, optionel): Couleur de l'intérieur du rectangle (``yellow`` par default)
    """

    couleur = rgb2hex(rgb(couleur))
    remplissage = rgb2hex(rgb(remplissage))


    global fenetre

    tab = [0]*9
    tab[0] = fenetre.create_rectangle(fenetre._conv(x1,y1),fenetre._conv(x1+largeur,y1+hauteur),
                                   outline = couleur, width = taille, fill= remplissage)
    tab[1] = 'rectangle'
    tab[2] = x1
    tab[3] = y1
    tab[4] = x1+largeur
    tab[5] = y1+hauteur
    tab[6] = couleur
    tab[7] = taille
    tab[8] = remplissage
    fenetre.objets.append(tab)


def affiche_fenetre():
    """Affiche la fenêtre graphique."""
    global fenetre
    fenetre.loop

### Fonction principale pour demarrer ####
def creer_fenetre(xmin=-10, xmax=10, ymin=-10, ymax=10, fond = 'blanc',titre = "Repère mathematique", axes = True):
    '''
    alias de fenetre()
    '''
    fenetre(xmin, xmax, ymin, ymax, fond,titre, axes)

def window(xmin=-10, xmax=10, ymin=-10, ymax=10, fond = 'blanc',titre = "Repère mathematique", axes = True):
    '''
    alias de fenetre()
    '''
    fenetre(xmin, xmax, ymin, ymax, fond,titre, axes)

def fenetre(xmin=-10, xmax=10, ymin=-10, ymax=10, fond = 'blanc',titre = "Repère mathematique", axes = True):
    """Initialise l'object fenetre graphique sans l'afficher.

    Arguments:
        titre (str): Titre de la fenêtre. La valeur par défaut est ``Repère mathematique``.
        xmin,xmax,ymin,ymax (float) : Dimensions du repère. Les valeurs par défaut sont -10, 10, -10, 10
        fond (`couleur <#couleurs>`_, optionel): Couleur de fond de la fenêtre (``blanc`` par défaut).
        axes (bool, optionel): Affiche les axes du repère si ``True`` (``True`` par défaut).
    """

    fond = rgb2hex(rgb(fond))

    root = Tk()       # on crée la fenêtre
    root.title(titre) # on lui donne un titre
    graph = _Fenetre_graphique(root,xmin = xmin, xmax=xmax, ymin = ymin, ymax=ymax,
                              background = fond, axes = axes)
    graph.pack(side=LEFT,fill=BOTH, expand= YES)
    global fenetre
    fenetre = graph
    return graph



"""
Partie trigonométrie du module lycee.
Deux familles de fonctions: ``_degré`` et ``_radian``.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

import math


def cos_radian(angle):
    """
    Renvoie le cosinus de l'angle.

    Arguments:
        angle (float): La mesure d'un angle en radians
    """
    return math.cos(angle)


def sin_radian(angle):
    """
    Renvoie le sinus de l'angle.

    Arguments:
        angle (float): La mesure d'un angle en radians
    """
    return math.sin(angle)


def tan_radian(angle):
    """
    Renvoie la tangente de l'angle.

    Arguments:
        angle (float): La mesure d'un angle en radians
    """
    return math.tan(angle)


def acos_radian(x):
    """
    Renvoie un angle en radians dont le cosinus vaut ``x``.

    Arguments:
        x (float): Un nombre entre -1 et 1
    """
    return math.acos(x)


def asin_radian(x):
    """
    Retourne un angle en radians dont le sinus vaut ``x``.

    Arguments:
        x (float): Un nombre entre -1 et 1
    """
    return math.asin(x)


def atan_radian(x):
    """
    Renvoie un angle en radians dont la tangente vaut ``x``.

    Arguments:
        x (float): Un nombre entre -1 et 1
    """
    return math.atan(x)


def cos_degre(angle):
    """
    Renvoie le cosinus de l'angle (type``float``).

    Arguments:
        angle (float): La mesure d'un angle en degré
    """
    return math.cos(math.radians(angle))


def sin_degre(angle):
    """
    Renvoie le sinus de l'angle (type``float``).

    Arguments:
        angle (float): La mesure d'un angle en degré
    """
    return math.sin(math.radians(angle))


def tan_degre(angle):
    """
    Renvoie la tangente de l'angle (type``float``).

    Arguments:
        angle (float): La mesure d'un angle en degré.
    """
    return math.tan(math.radians(angle))


def acos_degre(x):
    """
    Renvoie un angle en degré (float) dont le cosinus vaut ``x``.

    Arguments:
        x (float): Un nombre réel
    """
    return math.degrees(math.acos(x))


def asin_degre(x):
    """
    Renvoie un angle en degrés (foat) dont le sinus vaut ``x``.

    Arguments:
        x (float): Un nombre réel
    """
    return math.degrees(math.asin(x))


def atan_degre(x):
    """
    Renvoie un angle en degrés (foat) dont le sinus vaut ``x``.

    Arguments:
        x (float): un nombre réel
    """
    return math.degrees(math.atan(x))



# -*- coding: utf-8 -*-

import tkinter as tk

def demander_texte(titre="Entrez un texte", message=None):
    """
    Ouvre une fenêtre avec le titre "titre" et attend une chaine de caractères.

    Arguments:
        titre (str, optionel): Le titre de la fenetre (``"Entrez un texte"`` par défaut).
        message (str, optionel): Si présente, on ajoute un champ de texte contenant ``message``.

    Returns:
        La chaine de caractère (type ``str``) entrée par l'utilisateur.
    """
    def _sauver_valeur(event=None):#la fonction sauve la valeur dans value et ferme la fenetre
        value.set(entree.get())
        fenetre.destroy()

    fenetre = tk.Tk()
    fenetre.title(titre)

    # Si message est entré, on ajoute un champ message.
    # permet par exemple l'affichage d'érreurs.
    if message is not None:
        label = tk.Label(fenetre, text = message)
        label.pack()

    value = tk.StringVar()

    entree = tk.Entry(fenetre, textvariable=value, width=50)
    entree.pack()
    # la touche retour appelle _sauver_valeur
    entree.bind('<Key-Return>', _sauver_valeur)
    # Donne le focus à la fenetre et au widget entree
    # Peut-etre focus_set serait plus poli?
    # http://tkinter.fdex.eu/doc/uwm.html#focus_set
    # http://tkinter.fdex.eu/doc/uwm.html#focus_force
    entree.focus_force()

    bouton=tk.Button(fenetre,text='Valider',command=_sauver_valeur)
    bouton.pack()

    fenetre.mainloop()
    return value.get()

def demander_reel(titre="Entrez un nombre réel"):
    """Ouvre une fenetre et attend un nombre réel.

    Si ce n'est pas un nombre réel, on repose la question en ajoutant un message d'erreur.

    Arguments:
        titre (str, optionel): Titre de la fenetre (``"Entrez un nombre réel"`` par défaut).

    Returns:
        Le nombre réel entré par l'utilisateur (type ``float``).
    """
    message = None
    while True:
        texte = demander_texte(titre, message)
        try:
            reel = float(texte)
        except ValueError:
            message = "La donnée est incorrecte. Ce n'est pas un nombre"
            continue
        else:
            break
    return reel

def demander_entier(titre="Entrez un nombre entier"):
    """Ouvre une fenetre et attend un nombre entier.

    Si ce n'est pas un nombre entier, on repose la question en ajoutant un message d'erreur.

    Arguments:
        titre (str, optionel): Titre de la fenêtre (``"Entrez un nombre entier"`` par défaut).

    Returns:
        Le nombre entier entré par l'utilisateur (type ``float``).
    """
    message = None
    while True:
        texte = demander_texte(titre, message)
        try:
            entier = int(texte)
        except ValueError:
            message = "La donnée est incorrecte. Ce n'est pas un nombre ENTIER"
            continue
        else:
            break
    return entier




def pgcd(a,b):
    """Renvoie le Plus Grand Diviseur Communs  des entiers ``a`` et ``b``.
    
    Arguments:
        a (int) : un nombre entier
        b (int) : un nombre entier
    """
    if a<0 or b<0:
        return pgcd(abs(a),abs(b))
    if b==0:
        if a==0:
            raise DivisionByZeroError("Le PGCD de deux nombres nuls n'existe pas")
        else:
            return a
    else:
        return pgcd(b,a%b)

def reste(a,b):
    """Renvoie le reste de la division de ``a`` par ``b``.

    Arguments:
        a (int): Un nombre entier.
        b (int): Un nombre entier non nul.
    """
    r=a%b
    if r<0 : r=r+abs(b)
    return r

def quotient(a,b):
    """Le quotient de la division de ``a`` par ``b``.

    Arguments:
        a (int): Un nombre entier.
        b (int): Un nombre entier non nul.
    """
    return a//b



# -*- coding: utf-8 -*-
"""
Gestion des couleurs en Francais.

Permet la conversion francais -> rgb <-> hexadécimal.
Utilisé par les librairies repere et graphique

Notre fonction principale rgb(c) retourne un tuple RGB.
En gros on accepte un str du type "rouge", un str en hexadécimal ou un tuple RGB et on sort un RGB.

L'utilisateur peut ensuite choisir de le convertir en hexadécimal avec rgb2hex si besoin.

Licence MIT.
"""

_couleurs = {
    'noir': (0, 0, 0),
    'blanc': (255, 255, 255),
    'gris': (128, 128, 128),
    'rouge': (255, 0, 0),
    'vert': (0, 255, 0),
    'bleu': (0, 0, 255),
    'jaune': (255, 255, 0),
    'cyan':  (0, 255, 255),
    'magenta':(255, 0, 255),
    'orange': (255, 165, 0),
}

def rgb2hex(rgb):
    """
    Convertit un tuple RGB en hexadécimal
    """
    return '#%02x%02x%02x' % rgb

def hex2rgb(hex):
    """
    Convertit une coulur hexadécimale en tuple RGB
    """
    h = hex.lstrip("#")
    return tuple(int(h[i:i+2], 16) for i in (0, 2 ,4))

def rgb(c):
    """
    On formate une couleur en RGB.

    Arguments:
        c (str ou tuple): Couleur sous la form "rouge", hexa ou rgb

    Returns:
        Tuple RGB.
    """

    if isinstance(c, tuple):
        return c
    elif isinstance(c, str):
        if c.startswith("#"):
            # On a un format hexadécimal
            return hex2rgb(c)
        if c.lower() in _couleurs:
            # on a un str
            return _couleurs[c]
    # Au pire, on retourne c et on récupérera une erreur tkinter ou pygame
    return c



"""
Partie listes du module lycee.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

from chaines import fich2chaine, chaine2fich


def CSV2liste(num, fichier='optionel'):
    """
    Retourne une liste correspondant à la colonne ou la ligne nom du fichier ``fichier``.

    Si fichier n'est pas précisé, ouvre une boite de dialogue pour le choisir
    Le fichier ne doit contenir que des nombres et le séparateur doit être ``;``

    Arguments:
        num (str ou int): Un numéro de ligne ou un nom de colonne (``A`` à ``Z`` ).
        fichier (file,optionel): Le nom complet (avec le chemin) d'un fichier contenant des nombres.
    """
    ch = fich2chaine(fichier)
    if type(num) == int:
        L = ch.split("\n")
        if len(L) >= num:
            R = []
            for n in L[num - 1].split(";"):
                try:
                    R.append(eval(n))
                except:
                    raise Exception("Problème lors de l'importation")
            return R
    if type(num) == str:
        num = num.upper()
        c = ord(num) - ord('A')
        R = []
        for m1 in ch.split("\n"):
            m2 = m1.split(";")
            if len(m2) > c and m2[c] != '':
                try:
                    R.append(eval(m2[c].replace(' ', '').replace(',', '.')))
                except:
                    raise Exception("Problème lors de l'importation")
        return R

def liste2CSV(L, fichier='optionel'):
    """
    Enregistre sous le nom ``fichier`` la liste ``L``.

    Si fichier n'est pas précisé, ouvre une boite de dialogue pour le choisir

    Arguments:
        L (list): Une liste
        fichier (file, optionel): Le nom complet (avec le chemin) d'un fichier contenant du texte brut.
    """
    for i in range(len(L)):
        L[i] = str(L[i])
    chaine2fich("\n".join(L), fichier)
liste2CSV([1,'a',3,'bonjour',10,'aurevoir'], fichier='TableurPourTest2.csv')

def trier(liste1,liste2=[]):
    """
    Retourne `liste1` triée.
    Si `liste2` est renseignée, elle est réorganisée de la même manière que `liste1`.
    Ex 1 : trier([5,3,4])=[3,4,5]

    Ex 2 : `liste1=[3,2,1]` et `liste2=[10,20,30]`.
    trier(liste1,liste2)=([1,2,3],[30,20,10])

    Arguments:
        liste1 (list) : Une liste avec UNIQUEMENT des nombres OU UNIQUMENT des chaines de caractères
        liste2 (list,optionnel) : Une liste quelconque mais de même taille que `liste1`
    """

    if liste2==[]:
        return sorted(liste1)
    elif len(liste1)!=len(liste2):
        print("Erreur : les 2 listes n'ont pas la même taille !")
    else:
        #Il faut trier la liste selon l'ordre de liste_groupe
        #On crée une matrice n x 2
        L=[]
        for i in range(len(liste1)):
            L.append([liste1[i],liste2[i]])
        L.sort()
        return [i for i,j in L],[j for i,j in L]

def transposer(L):
    """
    L est une liste de liste, comme une matrice NxM. On prend la transposée.
    Ex : L=[[a,b,c],[1,2,3]]. Retourne la liste [[a,1],[b,2],[c,3]]

    Arguments:
        L (list) : une liste de listes
    """
    n=len(L[0])
    for i in range(len(L)):
        if n!=len(L[i]):
            print("Les listes n'ont pas la même taille !")

    return list(map(list, zip(*L)))

def serie(deb,fin,pas=1):
    """
    Renvoie une liste de nombre (float) de `deb` à `fin` (inclu) avec un `pas`

    Arguments
        deb (float ou int) : début de la série
        fin (float ou int) : fin de la série
        pas (float ou int, optionnel) : pas de la série
    """
    l=[deb,fin,pas]
    degmax=10**(-min([partie_entiere(log(abs(float(i)))) for i in l]))
    ToutEntier=[int(i*degmax) for i in l]
    return [float(i)/degmax for i in range(ToutEntier[0],ToutEntier[1]+ToutEntier[2],ToutEntier[2])]


def affiche_poly(L):  # Pourquoi???
    """
    Affiche la liste L sous forme d'un polynôme (L[n] étant le coefficient de degré n).

    Arguments:
        L (list): Une liste
    """
    poly = ""
    for i in range(len(L)):
        c = L[i]
        if c != 0 and poly != "":
            poly = poly + '+'
        if c != 0:
            if i > 0:
                if c == -1:
                    poly = poly + '-'
                if abs(c) != 1:
                    poly = poly + str(c)
            else:
                poly = poly + str(c)
            if i >0:
                poly = poly + 'X'
                if i > 1:
                    poly = poly + '^' + str(i)
    if poly == "":
        poly = 0
    return poly



"""
Partie chaines de caractères du module lycee.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

import tkinter as Tk
import tkinter.filedialog as tkf
import builtins


def taille(objet):
    """
    Retourne la longueur de cette chaine ou de cettte liste.

    Arguments:
        objet (str ou list): Une chaine de caractères ou une liste.
    """
    return builtins.len(objet)


def fich2chaine(fichier='optionel'):
    """
    Retourne chaine formée du contenu du fichier ``fichier``.

    Si ``fichier`` n'est pas précisé, ouvre une boite de dialogue pour le sélectionner.

    Arguments:
        fichier (file, optionel): Nom complet (avec le chemin) d'un fichier contenant du texte brut.

    """
    if fichier == 'optionel':
        fen = Tk.Tk()
        tex1 = Tk.Label(fen, text='Vous n''avez pas précisé de fichier')
        tex1.pack()
        fich = tkf.askopenfile(parent=fen, mode='rb', title="Choisissez un fichier")
        try:
            fen.destroy()
        except:
            pass
        if fich is not None:
            fichier = fich.name
    if fichier != 'optionel':
        filin = open(fichier, 'r')
        return "\n".join([line.strip() for line in filin])
        filin.close()
    else:
        return ""


def chaine2fich(ch, fichier='optionel'):
    """
    Enregistre sous le nom ``fichier`` la chaine ``ch``.

    Si fichier n'est pas précisé, ouvre une boite de dialogue pour le sélectionner.

    Arguments:
        ch (str): Une chaine de caractères
        fichier (file, optionel): Le nom complet (avec le chemnin) d'un fichier contenant du texte brut.
    """
    if fichier == 'optionel':
        fen = Tk.Tk()
        tex1 = Tk.Label(fen, text='Vous n''avez pas précisé de fichier')
        tex1.pack()
        fich = tkf.asksaveasfile(parent=fen, mode='w', title="Choisissez un fichier")
        try:
            fen.destroy()
        except:
            pass
        if fich is not None:
            fichier = fich.name
    if fichier != 'optionel':
        filout = open(fichier, 'w')
        filout.write(ch)
        filout.close()
        return True
    else:
        return False



# -*- coding: utf-8 -*-
#

"""
Partie vecteurs du module lycee.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

from math import sqrt


def vecteur(x, y, z=None):
    """
    Renvoie un vecteur de coordonées ``(x,y)`` ou ``(x,y,z)``.

    Arguments:
        x (float): Abscisse du vecteur
        y (float): Ordonnée du vecteur
        z (float, optionel): Cote du vecteur.

    """
    if z is None:
        return [x, y]
    else:
        return [x, y, z]


def norme(v):
    """
    Renvoie la norme du vecteur ``v``.

    Arguments:
        v (array): Un vecteur du plan ou de l'espace
    """

    n = 0
    for i in v:
        n = n + i*i
    return sqrt(n)


def abscisse(v):
    """
    Renvoie l'abscisse du vecteur ``v``.
        Arguments:
            v (array): Un vecteur du plan ou de l'espace
    """
    return v[0]


def ordonnee(v):
    """
    Renvoie l'ordonnée du vecteur ``v``.

    Arguments:
        v (array): Un vecteur du plan ou de l'espace
    """
    return v[1]


def cote(v):
    """
    Renvoie la cote du vecteur ``v``.

    Arguments:
        v (array): Un vecteur de l'espace
    """
    return v[2]



# -*- coding: utf-8 -*-
"""
Partie statistiques et probobilité du module lycee.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

import math
import builtins
from arithmetique import quotient
import random
from listes import transposer

# Loi Binomial
def binomial(n, p):
    """
    Renvoie un entier (int) représentant le coefficient binomial ``p`` parmi ``n``.

    C'est à dire le nombre de chemins de l’arbre réalisant ``p`` succès pour ``n`` répétitions.

    Arguments:
        n (int): Un nombre entier
        p (int): Un nombre entier
    """
    if p <= n:
        return quotient(
            math.factorial(n),
            math.factorial(n-p) * math.factorial(p)
        )
    else:
        return 0


def tirage_binomial(n, p):
    """
    Renvoie un nombre entier (int) choisi de manière aléatoire selon une loi binomiale B(n,p) : ``p`` parmi ``ǹ``.

    Arguments:
        n (int): Premier paramètre de la loi binomiale à simuler.
        p (int): Second paramètre de la loi binomiale à simuler.
    """
    s = 0
    for i in range(n):
        if alea() < p:
            s = s + 1
    return s

# Loi uniforme
def alea_entier(min, max):
    """
    Renvoie un entier (int) choisi de manière (pseudo)aléatoire et équiprobable
    dans l'intervalle [``min`` ; ``max``].

    Arguments:
        min (int): Un nombre entier
        max (int): Un nombre entier
    """
    return random.randint(min, max)

def tirage_uniforme(min, max):
    """
    Renvoie un nombre décimal (float) choisi de manière (pseudo)aléatoire et uniforme
    de l'intervalle \[``min`` ; ``max``\[.

    Arguments:
        min (float): Un nombre réel.
        max (float): Un nombre réel.

    """
    return random.uniform(min, max)

def choix(liste):
    """
    Renvoie un élément de la liste ``liste`` choisi (pseudo)aléatoirement et de manière équipropable

    Arguments:
        liste (int): La liste dans laquelle on choisit un élément.
    """
    return random.choice(liste)


def alea():
    """
    Renvoie au hasard un décimal de l'intervalle [0 ; 1[
    """
    return random.random()

# Loi exponentielle

def tirage_expo(x):
    """
    Renvoie un nombre décimal (float) choisi de manière aléatoire selon
    une loi exponentielle de paramètre ``x``.

    Arguments:
        x (float) : est un réel strictement positif.
    """
    return random.expovariate(x)

# Loi normale

def tirage_normale(mu, sigma):
    """
    Renvoie un nombre décimal (float) choisi de manière aléatoire
    selon une loi nomale d'espérance ``mu`` et d'écart type ``sigma``.

    Arguments:
        mu (float): Un nombre décimal. L'éspérance de la loi normale
        sigma (float): Un nombre décimal. L'écart type de la loi normale
    """
    return random.gauss(mu, sigma)

def tirage_gauss(mu,sigma):
    """
    Renvoie un nombre décimal (float) choisi de manière aléatoire
    selon une loi nomale d'espérance ``mu`` et d'écart type ``sigma``.

    Arguments:
        mu (float): Un nombre décimal. L'éspérance de la loi normale
        sigma (float): Un nombre décimal. L'écart type de la loi normale
    """
    return random.gauss(mu, sigma)

def compte(liste_critere,liste_effectif=[]):
    """
    Retourne la liste non triée, sans les doublons, les effectifs, les fréquences correspondantes et l'effectifs totales.
    La fonctionne NE TRIE PAS par ordre des critères : les critères n'ont pas nécessairement un ordre.
    Pour trier le résultat, utililser la fonctions trier()

    Arguments:
        liste_critere (list): une liste de critère d'une population
        liste_effectif (list, optionnel) : liste des frequences ou des effectifs des critères
    """
    n=len(liste_critere)
    eff_tot=0
    leff=[]
    if liste_effectif==[]: #On crée la liste des effectifs si elle n'existe pas, 1 par critère
        leff=[1]*n
    elif liste_effectif!=[] and len(liste_effectif)!=n:
        print("Erreur : les 2 listes n'ont pas le même nombre déléments !")
    else:
        leff=liste_effectif

    critere_groupe=[]
    eff_groupe=[]
    for i in range(n): #où l'on compte
        if liste_critere[i] not in critere_groupe:
            critere_groupe.append(liste_critere[i]) # Si le critere n'est pas dans la liste on le rajoute
            eff_groupe.append(leff[i]) # Et on ajoute l'effectif/frequence correspondante
            eff_tot+=leff[i]
        else:
            eff_groupe[critere_groupe.index(liste_critere[i])]+=leff[i] #Si le critere est déjà présent on ajoute l'effectif/frequence à celle déjà existante
            eff_tot+=leff[i]

    return [critere_groupe,eff_groupe,[float(i)/eff_tot for i in eff_groupe],eff_tot]


# Les classes
def centres(L):  # Pourquoi?
    """
    Renvoie une liste de longueur n-1 contenant les valeurs (L[i]+L[i+1])/2.

    Arguments:
        L (list): Une liste de taille n
    """
    R = []
    for i in range(len(L) - 1):
        R.append((L[i] + L[i + 1]) / 2)
    return R


def moyenne(xi, ni=[]):
    """
    Renvoie la moyenne de la liste ``xi``.

    Arguments:
        xi (list): liste de valeurs
        ni (liste, optionnel): série des effectifs ou des fréquences associés
    """
    L=compte(xi,ni)
    L1=transposer(L[::2]) #Manipulation technique pour ne prendre que critere et freq de la serie statistiques
                        # Et prendre la transposee
    m=0
    for i,j in L1:
        m+=i*j
    return m


def variance(xi,ni=[]):
    """
    Retourne la variance de la liste.

    Arguments:
        xi (list): Liste de valeurs
        ni (list, optionnel): Liste des effectifs associés
    """
    L=compte(xi,ni)
    m=moyenne(xi,ni)
    L1=transposer(L[::2]) #Manipulation technique pour ne prendre que critere et freq de la serie statistiques
                        # Et prendre la transposee
    v=0
    for i,j in L1:
        v+=(i-m)**2*j
    return v

def ecartype(xi, ni=[]):
    """
    Retourne l'écart-type de la liste.

    Arguments:
        xi (list): Liste de valeurs
        ni (list, optionnel): Liste des effectifs associés
    """
    return math.sqrt(variance(xi, ni))



# -*- coding: utf-8 -*-
#

"""
Fichier permettant de tester de façon systématique le bon fonctionnement des librairies
"""

from entree import *
from lycee import *
import pydiderotlibs.repere as repere
import pydiderotlibs.graphique as graphique
from time import sleep
import types
from tkinter import *
from couleurs import _couleurs, rgb, rgb2hex

def test_lycee():
    """
    Teste les fonctions de lycee
    """
    print('\n Les fonctions de lycee')
    # arithmetique
    print('pgcd(12,4) =',pgcd(12,4))
    print('pgcd(-4,12) = ',pgcd(-4,12))

    print('reste(15,4) = ',reste(15,4))

    print('quotient(15,4) = ',quotient(15,4))

    # fonctions usuelles

    print('4**1/2 = ',puissance(4,1/2))
    print('2**-2 = ',puissance(2,-2))

    print('carre =  ',carre(-2))

    print('racine de 4 = ',racine(4))

    print('4! = ',factoriel(4))

    print('Partie entière de -2,5 = ',partie_entiere(-2.5))
    print('Sans virgule = ',sans_virgule(-2.5))

    print('exp(1) = ',exp(1))
    print('ln(e) = ', ln(exp(1)))
    print('log(10) = ',log(10))


def test_chaines():
    print('\n Les fonctions sur les chaines')
    print('la fonction taille doit afficher 2 : ',taille('ab'))
    print('la fonction taille doit afficher 3 : ' ,taille([0,1,2]))

    chaine2fich('Yo !', 'blabla')
    print('Allez voir dans votre répertoire courant doit avoir un fichier blabla qui contient Yo !')
    input("Appuyez sur entrée pour continuer...")
    contenu=fich2chaine()
    print('Sélectionner le fichier blabla')
    print('La machine doit vous dire bonjour...')
    print('Machine : ',contenu)


def test_listes():
    print('\n Les fonctions sur les listes à partir de fichier')

    print('On doit avoir une liste de 0 à 9 : ',CSV2liste('A','TableurPourTest.csv'))
    print('On doit avoir une liste de 10 en 10 : ',CSV2liste(1,'TableurPourTest.csv'))

    liste2CSV([1,'a',3,'bonjour',10,'aurevoir'], fichier='TableurPourTest2.csv')
    print('Votre répertoire courant doit contenir un fichier TableurPourTest2.csv')

    x=[[1,2,'a'],[3,4,'b'],[5,6,'c'],[7,8,'d']]
    print(x)
    print("On transpose la liste ci-dessus : ",transposer(x))

    print("On affiche un polynome à partir d'une liste ",affiche_poly([1,1,1]))


def test_stats_proba():

    print('\n Les fonctions sur les stats et proba')

    print("Le nombre de choix de 2 parmi 5 : ",binomial(5, 2))

    print("Un nombre au hasard avec la loi binomiale de 2 parmi 5: ",tirage_binomial(5, 2))

    print("Un nombre entier au hasard entre -10 et 10 : ",alea_entier(-10, 10))

    print("Un nombre au hasard entre -5 et 5 avec la loi uniforme : ",tirage_uniforme(-5, 5))

    print(range(0,11))
    print("Un nombre au hasard dans la liste précédente : ",choix(contenu))

    print("Renvoie au hasard un décimal de l'intervalle [0 , 1[ : ",alea())

    print("Un nombre au hasard avec la loi exponentielle de paramètre 1 : ",tirage_expo(1))

    print("Un nombre au hasard avec la loi normale de paramètre 0 et 1 : ",tirage_normale(0, 1))

    ListeTest=[3,2,3,4,5,6,1,2,3,4,1,2,3]
    EffecTest=[1/55,2/55,3/55,4/55,5/55,6/55,7/55,8/55,9/55,1/55,2/55,3/55,4/55]
    print("Criteres : ",ListeTest," avec les effectifs : ",EffecTest)
    c=compte(ListeTest,EffecTest)
    print("Compte la liste ci-dessus : ",compte(ListeTest,EffecTest))
    print("On range : ",trier(c[0],c[1]),"\n")

    ListeTest2=['bla',2,3,4,5,[5,6],'bla',2,3,4,'bla',2,3]
    EffecTest2=[1,2,3,4,5,6,7,8,9,1,2,3,4]
    print("Criteres : ",ListeTest2," avec les effectifs : ",EffecTest2)
    print("Compte la liste ci-dessus : ",compte(ListeTest2,EffecTest2))

    print(range(0,11))
    print("Trouve le centre de la liste précédente : ",centres(range(0,11)))

    L=[float(i)  for i in range(100)]
    print("\n ------------------------------------\n",L)
    print("Moyenne de la liste ci-dessus : ",moyenne(L))
    print("Variance de la liste ci-dessus : ",variance(L))
    print("Ecart-type de la liste ci-dessus : ",ecartype(L))


def test_entree_tk():
    print('\n Les fonctions de entre_tk')
    nom = demander_texte('Bonjour', "Quel est ton nom?")

    age = demander_reel("Quel age as tu?")
    annee = str(int(2019 - age))

    print(nom + " est né(e) en " + annee)


def test_repere():
    # On initialise la fenetre
    repere.creer_fenetre()

    # On créé des objects geométriques
    repere.trace_point(5,3)
    repere.trace_segment(-10,-4,8,7,couleur='rouge', taille = 2)
    repere.trace_rectangle(1,1,8,4,couleur='noir', taille = 4, remplissage='jaune')
    repere.trace_point(5,5,couleur='bleu',taille=5)
    repere.trace_point(6,7,couleur='bleu',taille=5,forme='croix')
    repere.trace_texte(-3,3,"Un texte",couleur = 'bleu')

def test_graphique():
    # On créé la fenêtre graphique
    graphique.creer_fenetre()
    x = 100
    y = 100
    # Boucle principale
    while 1:
        evenements = graphique.demande_evenements()

        if 'souris' in evenements:
            [x,y] = evenements['souris']

        if 'clic' in evenements:
            # ici evenements['clic'] est une liste [x, y]
            print('clic aux coordonées ' + str(evenements['clic']))

        # Trace un cercle au coordonnées (x,y)
        graphique.trace_cercle(x, y)
        # Attend un dixième de secondes
        sleep(0.1)
        # Efface le cercle
        graphique.trace_cercle(x, y, couleur='blanc')


def test_couleurs():
    """
    Affiche les couleurs disponibles.

    http://www.science.smith.edu/dftwiki/index.php/Color_Charts_for_TKinter
    """
    MAX_ROWS = 12
    FONT_SIZE = 30 # (pixels)
    root = Tk()
    root.title("Named colour chart")
    row = 0
    col = 0
    for c, t in _couleurs.items():
        texte = c + " " + str(t)
        if c in ['noir', 'bleu']:
            couleur = rgb2hex(rgb('blanc'))
        else:
            couleur = rgb2hex(rgb('noir'))
        e = Label(
            root,
            text=texte,
            background=rgb2hex(t),
            foreground=couleur,
            font=(None, -FONT_SIZE)
        )
        e.grid(row=row, column=col, sticky=E+W)
        row += 1
        if (row > MAX_ROWS):
            row = 0
            col += 1

    root.mainloop()

print("\n" * 10)
message = "Vous pouvez tester les différents modules avec les fonctions suivantes:\n"
fonctions = [f + "()" for f in dir() if f.startswith("test")]
message += "\n".join(fonctions)
print(message)



# -*- coding: utf-8 -*-
#

"""
Partie fonctions fonctions_usuelles du module lycee.
"""
"""
Créé à partir d' Edupython: http://edupython.tuxfamily.org/

Licence CECILL http://www.cecill.info/
"""

import math


def puissance(a, n):  # il faut ajouter la fonction carre -> vraiment? a*a est simple
    """
    Renvoie le nombre réel (float) \(a^n\).

    Arguments:
        a (float): un nombre décimal.
        b (float): un nombre décimal représentant l'exposant.

    """
    return a**n

def carre(a): # parce que...
    """
    Renvoie le carre d'un nombre reel (float)

    Arguments:
        a (float): un nombre décimal.

    """
    return a*a


def racine(x):
    """
    Renvoie le nombre réel (float) racine carré de ``x`` \(\sqrt{x}\).

    Arguments:
        a (float): un nombre décimal.
        b (float): un nombre décimal représentant l'exposant.
    """
    return math.sqrt(x)


def factoriel(n):  # Dans quel programme de lycée ?
    """
    Renvoie \(n! = n x (n-1) x ... x 3 x 2 x 1\)

    Arguments:
        n (int): Un nombre entier positif.
    """
    return math.factorial(n)


def partie_entiere(x):  # Utilité ? Concept de la partie entière difficile au lycée pour les nombres négatifs. Faire une fonction pour prendre la partie sans la virgule ?
    """
    Renvoie la partie entiere du nombre ``x``, c'est a dire le plus grand entier inferieur au reel ``x``.

    Arguments:
        x (float): Un nombre décimal.

    """
    return math.floor(x)

def sans_virgule(x):
    """
    Retourne la partie du nombre x sans sa partie décimale. Ex : -2.5 devient -2

    Arguments:
        x (float): Un nombre decimal.

    """
    return int(x)


def exp(x):
    """
    Renvoie l'image du nombre ``x`` par la fonction exponentielle: \(e^x\).

    Arguments:
        x (float): Un nombre decimal.
    """
    return math.exp(x)


def ln(x):
    """
    Renvoie l'image du nombre ``x`` par la fonction logarithme népérien: \(\ln x \).

    Arguments:
        x (float): Un nombre décimal strictement positif.
    """
    return math.log(x)

def log(x):
    """
    Renvoie l'image du nombre ``x`` par la fonction logarithme décimal: \(\log x \).

    Arguments:
        x (float): Un nombre décimal strictement positif.
    """
    return math.log10(x)



# -*- coding: utf-8 -*-
#
# Version 1.1
"""
Module avec les fonctions de la classe de Seconde 2018 pour le lycée diderot (marseille).
On prend comme fichier de départ le module de l'irem d'Amiens
http://download.tuxfamily.org/amienspython/lycee.py
Licence http://www.cecill.info/

Il y a des fonctions d'Amiens qui devaient être renommées / améliorées.
On nomme les fonctions en FRANCAIS.

Le module lycee est un module python réalisé par le groupe AMIENS PYTHON
a pour objectif de simplifier un certain nombre de manipulations
avec python au lycée (cosinus en degré, calcul d'une moyenne d'une liste,
représentation statistiques variées, ...)

Pour l'utiliser, il suffit d'ajouter en début de programme

from lycee import *
"""


import math

from arithmetique import *
from trigo import *
from stats_proba import *
from fonctions_usuelles import *
from vecteurs import *
from chaines import *
from listes import *

pi = math.pi
